unit f_rdbgrid2;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, Grids, PropSaveMain, PropSaveGrids, DB,
  DBGrids, rDBGrid, DBCtrls, rHTMLLabel, Mask, rDBComponents, rImprovedComps,
  Math;

type
  TrDBGrid2 = class(TForm)
    DBNavigator1: TDBNavigator;
    gr2: TrDBGrid;
    lblInfo: TrHTMLLabel;
    PropSaveMain1: TPropSaveMain;
    rDBGridsPropSave1: TrDBGridsPropSave;
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure lblInfoAnchorClick(Sender: TObject; LinkName: String);
    procedure gr2PickListMeasureItem(Control: TWinControl; Index: Integer; var Height: Integer);
    procedure gr2PickListDrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
    procedure gr2PickListSetSize(Sender: TObject; var NewWidth, NewHeight: Integer; var Resize: Boolean);
    procedure AfterUpdate(DataSet: TDataSet);
    procedure gr2GetLookupListItemHeight(Sender: TObject; Canvas: TCanvas; KeyField: TField; ListFields: string; var Height: Integer);
    procedure gr2GetLookupListItemProps(Sender: TObject; Canvas: TCanvas; RowIndex, ColIndex: Integer; Selected: Boolean; KeyField, ListField: TField;
      var Width: Integer; Height: Integer; var Text: string; ShowBmp: TBitmap; var DrawSeparatorTop, DrawSeparatorBottom: Boolean);
  private
  public
  end;

var
  rDBGrid2: TrDBGrid2;

implementation

uses dm_data, f_main, rDBGrid_DTEdit, rdlg, rtool;
{$R *.dfm}

procedure TrDBGrid2.FormShow(Sender: TObject);
begin
  DMData.qLookupProducts.Close;
  DMData.qLookupProducts.Open;
  DMData.AfterUpdate:=AfterUpdate;
  gr2.Resort;
  gr2.DropDownRowsCount:=10;
end;

procedure TrDBGrid2.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  DMData.qLookupProducts.Close;
  DMData.aqSuppliers.Close;
  DMData.aqCategories.Close;
  DMData.AfterUpdate:=nil;
end;

procedure TrDBGrid2.AfterUpdate(DataSet: TDataSet);
begin
  // reload picklist after each post = after changes in table
  gr2.LoadAutoPickList;
end;

procedure TrDBGrid2.lblInfoAnchorClick(Sender: TObject; LinkName: String);
begin
  ShowrDBGridEditor(gr2);
  gr2.Invalidate;
end;

procedure TrDBGrid2.gr2PickListMeasureItem(Control: TWinControl; Index: Integer; var Height: Integer);
begin
  // set bigger item height for StockLevel to make space for picture
  if (Control is TCustomListBox) and (gr2.SelectedField = DMData.qLookupProductsStockLevel) then
    Height:=(Control as TCustomListBox).Canvas.TextHeight('Wg') + 6;
end;

procedure TrDBGrid2.gr2PickListSetSize(Sender: TObject; var NewWidth, NewHeight: Integer; var Resize: Boolean);
var
  lb: TCustomListBox;
begin
  // set dropdown list height size for StockLevel according ListMeasureItem event
  if (Sender is TCustomListBox) and (gr2.SelectedField = DMData.qLookupProductsStockLevel) then
  begin
    lb:=Sender as TCustomListBox;
    NewHeight:=Min(lb.Items.Count, 15) * (lb.Canvas.TextHeight('Wg') + 6) + 4;
    Resize:=true;
  end;
  // set lookup list minimum width for Category
  if (Sender is TDBLookupControl) and (gr2.SelectedField = DMData.qLookupProductsCategory) then
  begin
    if NewWidth < 200 then
      NewWidth:=200;
    Resize:=true;
  end;
end;

procedure TrDBGrid2.gr2PickListDrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  lb: TCustomListBox;
  s: string;
  x, y, img: Integer;
begin
  // draw item for StockLevel
  if (Control is TCustomListBox) and (gr2.SelectedField = DMData.qLookupProductsStockLevel) then
  begin
    lb:=(Control as TCustomListBox);
    s:=lb.Items[Index];

    // init font and image index according item text
    if SameText(s, 'High') then
    begin
      img:=7;
      lb.Canvas.Font.Style:=[fsBold];
    end
    else
      if SameText(s, 'Low') then
      begin
        img:=5;
        lb.Canvas.Font.Color:=clRed;
      end
      else
        if s <> '' then
          img:=6
        else
          img:=-1;
    // draw text
    y:=(Rect.Bottom - Rect.Top - lb.Canvas.TextHeight(s)) div 2;
    lb.Canvas.TextRect(Rect, Rect.Left + 2, Rect.Top + y, s);
    // draw image
    if img >= 0 then
    begin
      x:=Rect.Right - Main.GridImg.Width - 4;
      y:=(Rect.Bottom - Rect.Top - Main.GridImg.Height) div 2;
      Main.GridImg.Draw(lb.Canvas, Rect.Left + x, Rect.Top + y, img);
    end;
  end;
end;

procedure TrDBGrid2.gr2GetLookupListItemHeight(Sender: TObject; Canvas: TCanvas; KeyField: TField; ListFields: string; var Height: Integer);
begin
  // set Lookup item heigh for Category (space for 2 lines)
  if SameText(ListFields, DMData.qLookupProductsCategory.LookupResultField) then
    Height:=Canvas.TextHeight('0') * 2 + 6;
end;

procedure TrDBGrid2.gr2GetLookupListItemProps(Sender: TObject; Canvas: TCanvas; RowIndex, ColIndex: Integer; Selected: Boolean; KeyField, ListField: TField;
  var Width: Integer; Height: Integer; var Text: string; ShowBmp: TBitmap; var DrawSeparatorTop, DrawSeparatorBottom: Boolean);
var
  id, x, y: Integer;
begin
  // set canvas property and image for Supplier lookup
  if SameText(ListField.FieldName, DMData.qLookupProductsSupplier.LookupResultField) then
  begin
    // set color for some records
    if AnsiPosEx('new', Text) > 0 then
      Canvas.Font.Color:=clRed;
    // define image
    ShowBmp.Transparent:=true;
    ShowBmp.Width:=Main.GridImg.Width;
    ShowBmp.Height:=Main.GridImg.Height;
    ShowBmp.Canvas.Brush.Color:=clWhite;
    ShowBmp.Canvas.FillRect(Classes.Rect(0, 0, ShowBmp.Width, ShowBmp.Height));
    Main.GridImg.GetBitmap(1, ShowBmp);
  end;

  // use own drawing for Category lookup
  if SameText(ListField.FieldName, DMData.qLookupProductsCategory.LookupResultField) then
  begin
    id:=KeyField.AsInteger;
    // fill rect
    Canvas.FillRect(Rect(0, 0, Width, Height));
    // own drawing 2 lines
    x:=Main.GridImg.Width + 6;
    y:=2;
    Canvas.TextOut(x, y, Text);
    Inc(y, Canvas.TextHeight('0') + 2);
    Canvas.Font.Color:=clGrayText;
    Canvas.TextOut(x, y, 'some description of cat. id ' + IntToStr(id));
    // draw image
    y:=(Height - Main.GridImg.Height) div 2;
    Main.GridImg.Draw(Canvas, 2, y, id);
    // show separator line
    DrawSeparatorBottom:=true;
    // set text='' to block default item drawing
    Text:='';
  end;
end;

end.
