unit dm_auxdata;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, WideStrings, DB, StdCtrls, DBCtrls, rHTMLLabel, ComCtrls,
  ToolWin, Grids, rDBRecView, CheckLst, Mask,
  rImprovedComps, rDBComponents, rDBComponents_UDB, PropSaveMain;

type
  TAuxData = class(TDataModule)
    PropSaveFile1: TPropSaveFile;
    procedure DataModuleCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    DefaultLabelsLanguage: string;
    FillLabelsTableByCaption: boolean;

    procedure LoadFieldsProps(DS: TDataSet);
    procedure DataSetCallBack(Sender: TObject);
    procedure ChangeLanguage(Lang: string = '');

    procedure SetAllChildrenDataSource(W: TwinControl; DS: TDataSource; ExcludeTag: integer);
  end;

var
  AuxData: TAuxData;

implementation

{$R *.dfm}
uses RosiCompGlobalCfg, dm_data, rdlg, rtool, f_products;

procedure TAuxData.DataModuleCreate(Sender: TObject);
begin
  // init global val
  DefaultLabelsLanguage:='EN';
  FillLabelsTableByCaption:=true;
  // link UDB LoadFieldsPropsProc to our LoadFieldsProps
  RosiCompGlobalCfg.RosiCompConfig.rDBFieldDefaultsCfg.LoadFieldsPropsProc:=LoadFieldsProps;
  // LoadINI file
  self.PropSaveFile1.LoadFile(GetAppDir+'UDBTest.ini');
end;

procedure TAuxData.LoadFieldsProps(DS: TDataSet);
var
  i: integer;
  F, L: TField;
  TblName, d, e: string;
  DefExist, appended: boolean;

  procedure SetFieldProps;
  begin
    DefExist:=true;
    L:=DMData.qCfgFldLabels.FieldByName('DISPLAY_LABEL_' + DefaultLabelsLanguage);
    if not L.IsNull then
      F.DisplayLabel:=L.AsString;
    L:=DMData.qCfgFldLabelsREQUIRED;
    if not L.IsNull then
      F.Required:=L.AsInteger=1;
    d:=DMData.qCfgFldLabelsDISPLAY_FORMAT.AsString;
    e:=DMData.qCfgFldLabelsEDIT_FORMAT.AsString;
    if d <> '' then
    begin
      if F is TNumericField then (F as TNumericField)
        .DisplayFormat:=d;
      if F is TBooleanField then (F as TBooleanField)
        .DisplayValues:=d;
      if F is TDateTimeField then (F as TDateTimeField)
        .DisplayFormat:=d;
    end;
    if e <> '' then
    begin
      if F is TNumericField then (F as TNumericField)
        .EditFormat:=e;
    end;
  end;

begin
  appended:=false;
  if DS is TQueryUDB then
    TblName:=(DS as TQueryUDB).TableName
  else
    TblName:='*';
  try
    // open fields definition if not active
    if (not DMData.qCfgFldLabels.Active) and DMData.qCfgFldLabels.Connection.Connected then
      DMData.qCfgFldLabels.RequeryEx;
    if (not DMData.qCfgFldLabels.Active) then
      Exit;
    // find definition for each field
    for i:=0 to DS.FieldCount - 1 do
    begin
      F:=DS.Fields[i];
      DefExist:=false;
      DMData.qCfgFldLabels.Locate(DMData.qCfgFldLabelsFIELD_NAME.FieldName, F.FieldName, [loCaseInsensitive]);
      repeat
        if SameText(DMData.qCfgFldLabelsFIELD_NAME.AsString, F.FieldName) then
        begin
          if SameText(DMData.qCfgFldLabelsTABLE_NAME.AsString, TblName) or SameText(DMData.qCfgFldLabelsTABLE_NAME.AsString, '*') then
            SetFieldProps;
          DMData.qCfgFldLabels.Next;
        end
        else
          Break;
      until DMData.qCfgFldLabels.Eof;

      if FillLabelsTableByCaption and not DefExist then
      begin
        // add field name to Cfg table
        appended:=true;
        DMData.qCfgFldLabels.Append;
        DMData.qCfgFldLabelsTABLE_NAME.AsString:='*';
        DMData.qCfgFldLabelsFIELD_NAME.AsString:=F.FieldName;
        DMData.qCfgFldLabels.FieldByName('DISPLAY_LABEL_' + DefaultLabelsLanguage).AsString:=F.FieldName;
        DMData.qCfgFldLabelsAUTO_ADDED.AsInteger:=1;
        DMData.qCfgFldLabels.Post;
      end;
    end;
  except
    on e: Exception do
    begin
      DMData.qCfgFldLabels.Cancel;
      DMData.qCfgFldLabels.Close;
      DlgE('Internal error during loading DisplayLabel for component ' + DS.Name + ' ' + e.Message);
    end;
  end;
  if appended then
    DMData.qCfgFldLabels.Close;
end;

procedure TAuxData.DataSetCallBack(Sender: TObject);
begin
  if Sender is TQueryUDB then (Sender as TQueryUDB)
    .LoadFieldsProps;
end;

procedure TAuxData.ChangeLanguage(Lang: string = '');
begin
  if Lang <> '' then
    DefaultLabelsLanguage:=Lang;
  DMData.Con1.CallProcForRegisteredDataset(DataSetCallBack);
  FreeF(Products);
end;

procedure TAuxData.SetAllChildrenDataSource(W: TwinControl; DS: TDataSource; ExcludeTag: integer);
  procedure ProcWinControl(WCont: TWinControl);
  var
    I: integer;
    C: TControl;
  begin
    for I:=0 to WCont.ControlCount - 1 do
    begin
      C:=WCont.Controls[I];
      if C.Tag = ExcludeTag then
        Continue;
      if C is TrDBMoveButton then
        TrDBMoveButton(C).DataSource:=DS;
      if C is TrDBToolButton then
        TrDBToolButton(C).DataSource:=DS;
      if C is TWinControl then
        ProcWinControl(C as TWinControl);
    end;
  end;

begin
  ProcWinControl(W);
end;

end.
